--LagBar by Derkyle

LagBar = {};
LagBar.version = GetAddOnMetadata("LagBar", "Version")
LagBar.PL_Lock = false

LagBar.LOW_LATENCY = 300
LagBar.MEDIUM_LATENCY = 600
LagBar.MAX_INTERVAL = 1
LagBar.UPDATE_INTERVAL = 0

function LagBar:Enable()

	if LagBar_DB == nil then
		LagBar_DB = {};
		LagBar_DB.x = 0;
		LagBar_DB.y = 0;
		LagBar_DB.locked = false;
	end

	LagBar:DrawGUI();
end

function LagBar:PlayerLogin()

	if LagBar.PL_Lock then return end

	LagBar.PL_Lock = true
	
		LagBarFrame:ClearAllPoints();
		LagBarFrame:SetPoint("CENTER", UIParent, LagBar_DB.x and "BOTTOMLEFT" or "BOTTOM", LagBar_DB.x or 0, LagBar_DB.y or 221);
		LagBarFrame:Show();

	LagBar.PL_Lock = false
	
end

function LagBar:DrawGUI()
	
	--don't repeat
	if LagBarFrame then
		return
	end
	
	--frames cannot have onclick handlers

	lbFrame = CreateFrame("Frame", "LagBarFrame", UIParent, "GameTooltipTemplate");
	lbFrame:SetPoint("CENTER", UIParent, LagBar_DB.x and "BOTTOMLEFT" or "BOTTOM", LagBar_DB.x or 0, LagBar_DB.y or 221);
	lbFrame:EnableMouse(true);
	lbFrame:SetToplevel(true);
	lbFrame:SetMovable(true);
	lbFrame:SetFrameStrata("LOW");
	lbFrame:SetWidth(120);
	lbFrame:SetHeight(25);

	lbFrame:RegisterForDrag("LeftButton")
	lbFrame:RegisterEvent("PLAYER_LOGIN");

	lbFrame.text = lbFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormalSmall");
	lbFrame.text:SetPoint("CENTER", lbFrame, "CENTER", 0, 0);
	lbFrame.text:Show();
			
		
		lbFrame:SetScript("OnEvent", function(frame, event, ...) 
		
			if event == "PLAYER_LOGIN" then
				LagBar:PlayerLogin();
			end
			
			end)
		
		lbFrame:SetScript("OnLoad", function()

			end)

		lbFrame:SetScript("OnShow", function()


			end)
			
		lbFrame:SetScript("OnUpdate", function()
			
				LagBar:OnUpdate(arg1);

			end)

		lbFrame:SetScript("OnMouseDown", function(frame, button) 
			
				if not LagBar_DB.locked and button ~= "RightButton" then
					frame.isMoving = true
					frame:StartMoving();
				end
			
			end)

		lbFrame:SetScript("OnMouseUp", function(frame, button) 

				if not LagBar_DB.locked and button ~= "RightButton" then
					if( frame.isMoving ) then

						frame.isMoving = nil
						frame:StopMovingOrSizing()

						LagBar_DB.x, LagBar_DB.y = frame:GetCenter()
					end
				
				elseif button == "RightButton" then
				
					if LagBar_DB.locked then
						LagBar_DB.locked = false
						DEFAULT_CHAT_FRAME:AddMessage("LagBar: Unlocked");
					else
						LagBar_DB.locked = true
						DEFAULT_CHAT_FRAME:AddMessage("LagBar: Locked");
					end
				end

			end)

end

function LagBar:OnUpdate(arg1)

	if (LagBar.UPDATE_INTERVAL > 0) then
		LagBar.UPDATE_INTERVAL = LagBar.UPDATE_INTERVAL - arg1;
	else
		LagBar.UPDATE_INTERVAL = LagBar.MAX_INTERVAL;
		
		--local bandwidthIn, bandwidthOut, latency = GetNetStats();
		--if (latency > LagBar_MEDIUM_LATENCY) then
		--	LagBar_Text:SetTextColor(1, 0, 0);									
		--elseif (latency > LagBar_LOW_LATENCY) then
		--	LagBar_Text:SetTextColor(1, 1, 0);
		--else
		--	LagBar_Text:SetTextColor(0, 1, 0);
		--end
		--if (latency > 9999) then
		--	LagBar_Text:SetText("Ping: HIGH");
		--else
		--	LagBar_Text:SetText("Ping: "..latency.." ms");
		--end

		local framerate = floor(GetFramerate() + 0.5)
		local framerate_text = format("|cff%s%d|r fps", LagBar_GetThresholdHexColor(framerate / 60), framerate)
		
		local latency = select(3, GetNetStats())
		local latency_text = format("|cff%s%d|r ms", LagBar_GetThresholdHexColor(latency, 1000, 500, 250, 100, 0), latency)
		
		LagBarFrameText:SetText(framerate_text.." | "..latency_text);
	end
end


function LagBar_GetThresholdHexColor(quality, ...)
	local r, g, b = LagBar_GetThresholdColor(quality, ...)
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end


function LagBar_GetThresholdColor(quality, ...)

	local inf = 1/0
	
	if quality ~= quality or quality == inf or quality == -inf then
		return 1, 1, 1
	end

	local percent = LagBar_GetThresholdPercentage(quality, ...)

	if percent <= 0 then
		return 1, 0, 0
	elseif percent <= 0.5 then
		return 1, percent*2, 0
	elseif percent >= 1 then
		return 0, 1, 0
	else
		return 2 - percent*2, 1, 0
	end
end

function LagBar_GetThresholdPercentage(quality, ...)
	local n = select('#', ...)
	if n <= 1 then
		return LagBar_GetThresholdPercentage(quality, 0, ... or 1)
	end

	local worst = ...
	local best = select(n, ...)

	if worst == best and quality == worst then
		return 0.5
	end

	if worst <= best then
		if quality <= worst then
			return 0
		elseif quality >= best then
			return 1
		end
		local last = worst
		for i = 2, n-1 do
			local value = select(i, ...)
			if quality <= value then
				return ((i-2) + (quality - last) / (value - last)) / (n-1)
			end
			last = value
		end

		local value = select(n, ...)
		return ((n-2) + (quality - last) / (value - last)) / (n-1)
	else
		if quality >= worst then
			return 0
		elseif quality <= best then
			return 1
		end
		local last = worst
		for i = 2, n-1 do
			local value = select(i, ...)
			if quality >= value then
				return ((i-2) + (quality - last) / (value - last)) / (n-1)
			end
			last = value
		end

		local value = select(n, ...)
		return ((n-2) + (quality - last) / (value - last)) / (n-1)
	end
end
